@@ -199,7 +199,6 @@ def read(filenames, start_date=None, end_date=None, dense=False,
     for index, filename in enumerate(filenames):
         tsd = tsutils.date_slice(tsutils.read_iso_ts(filename, dense=dense),
                                  start_date=start_date, end_date=end_date)
-        tsd.dropna(how='all', inplace=True)
         try:
             result = result.join(tsd,
                                  how=how,
@@ -1177,7 +1176,7 @@ def plot(
         norm_xaxis=False,
         norm_yaxis=False,
         xy_match_line='',
-        grid=True,
+        grid=None,
         input_ts='-',
         start_date=None,
         end_date=None,
@@ -1367,7 +1366,7 @@ def plot(
        DEPRECATED: use '--type="norm_xaxis"' or '--type="norm_yaxis"' instead.
     :param xy_match_line: Will add a match line where x == y.  Default is ''.
        Set to a line style code.
-    :param grid: boolean, default False
+    :param grid: boolean, default True
        Whether to plot grid lines on the major ticks.
     :param input_ts: Filename with data in 'ISOdate,value' format or '-' for
        stdin.
@@ -1523,7 +1522,7 @@ def plot(
 *
 '''.format(xaxis))
 
-    if type == 'norm_yaxis': 
+    if type == 'norm_yaxis':
         yaxis = 'normal'
         if yaxis != 'arithmetic':
             import warnings
@@ -1754,11 +1753,167 @@ def plot(
 *
 '''.format(type))
 
+    if grid is None:
+        grid = True
+    else:
+        grid = False
     plt.grid(grid)
     plt.title(title)
     plt.savefig(ofilename)
 
 
+def _dtw(ts_a, ts_b, d = lambda x,y: abs(x-y), window=10000):
+    """Returns the DTW similarity distance between two 2-D
+    timeseries numpy arrays.
+
+    Arguments
+    ---------
+    ts_a, ts_b : array of shape [n_samples, n_timepoints]
+        Two arrays containing n_samples of timeseries data
+        whose DTW distance between each sample of A and B
+        will be compared
+
+    d : DistanceMetric object (default = abs(x-y))
+        the distance measure used for A_i - B_j in the
+        DTW dynamic programming function
+
+    Returns
+    -------
+    DTW distance between A and B
+    """
+
+    # Create cost matrix via broadcasting with large int
+    ts_a, ts_b = pd.np.array(ts_a), pd.np.array(ts_b)
+    M, N = len(ts_a), len(ts_b)
+    cost = sys.maxint * pd.np.ones((M, N))
+
+    # Initialize the first row and column
+    cost[0, 0] = d(ts_a[0], ts_b[0])
+    for i in xrange(1, M):
+        cost[i, 0] = cost[i-1, 0] + d(ts_a[i], ts_b[0])
+
+    for j in xrange(1, N):
+        cost[0, j] = cost[0, j-1] + d(ts_a[0], ts_b[j])
+
+    # Populate rest of cost matrix within window
+    for i in xrange(1, M):
+        for j in xrange(max(1, i - window),
+                        min(N, i + window)):
+            choices = cost[i - 1, j - 1], cost[i, j-1], cost[i-1, j]
+            cost[i, j] = min(choices) + d(ts_a[i], ts_b[j])
+
+    print(cost)
+    # Return DTW distance given window
+    return cost[-1, -1]
+
+@mando.command
+def dtw(window=10000,
+        input_ts='-',
+        start_date=None,
+        end_date=None):
+    tsd = tsutils.date_slice(tsutils.read_iso_ts(input_ts, dense=False),
+                             start_date=start_date,
+                             end_date=end_date)
+
+    process = {}
+    for i in tsd.columns:
+        for j in tsd.columns:
+            if (i, j) not in process and (j, i) not in process and i != j:
+                process[(i, j)] = _dtw(tsd[i], tsd[j], window=window)
+
+    print(process.keys())
+
+    ntsd = pd.DataFrame(process.values(), process.keys())
+    return tsutils.printiso(ntsd)
+
+@mando.command
+def pca(n_components=None,
+        input_ts='-',
+        start_date=None,
+        end_date=None):
+    '''
+    Returns the principal components analysis of the time series.  Does not
+    return a time-series.
+
+    :param n_components: The number of groups to separate the time series into.
+    :param input_ts: Filename with data in 'ISOdate,value' format or '-' for
+       stdin.
+    :param start_date: The start_date of the series in ISOdatetime format, or
+        'None' for beginning.
+    :param end_date: The end_date of the series in ISOdatetime format, or
+        'None' for end.
+    '''
+    from sklearn.decomposition import PCA
+
+    tsd = tsutils.date_slice(tsutils.read_iso_ts(input_ts),
+                             start_date=start_date,
+                             end_date=end_date)
+
+    pca = PCA(n_components)
+    pca.fit(tsd.dropna(how='any'))
+    print(pca.components_)
+
+
+@mando.command
+def normalization(mode='minmax',
+                  min_limit=0,
+                  max_limit=1,
+                  pct_rank_method='average',
+                  print_input=False,
+                  float_format='%g',
+                  input_ts='-',
+                  start_date=None,
+                  end_date=None):
+    '''
+    Returns the normalization of the time series.
+
+    :param mode: 'minmax' or 'zscore'.  Default is 'minmax'
+                 'minmax' is (X-Xmin)/(Xmax-Xmin).
+                 'zscore' is X-mean(X)/stddev(X)
+                 'pct_rank' is rank(X)*100/N
+    :param min_limit: Defaults to 0.  Defines the minimum limit of
+        the minmax normalization.
+    :param max_limit: Defaults to 1.  Defines the maximum limit of
+        the minmax normalization.
+    :param pct_rank_method: Defaults to 'average'.  Defines how tied ranks
+        are broken.  Can be 'average', 'min', 'max', 'first', 'dense'.
+    :param print_input: If set to 'True' will include the input
+        columns in the output table.  Default is 'False'.
+    :param input_ts: Filename with data in 'ISOdate,value' format or '-' for
+       stdin.
+    :param start_date: The start_date of the series in ISOdatetime format, or
+        'None' for beginning.
+    :param end_date: The end_date of the series in ISOdatetime format, or
+        'None' for end.
+    '''
+    tsd = tsutils.date_slice(tsutils.read_iso_ts(input_ts),
+                             start_date=start_date,
+                             end_date=end_date)
+
+    # Trying to save some memory
+    if print_input:
+        otsd = tsd.copy()
+    else:
+        otsd = pd.DataFrame()
+
+    if mode == 'minmax':
+        tsd = min_limit + (tsd - tsd.min())/(tsd.max() - tsd.min())
+    elif mode == 'zscore':
+        tsd = (tsd - tsd.mean())/tsd.std()
+    elif mode == 'pct_rank':
+        tsd = tsd.rank(method=pct_rank_method, pct=True)
+    else:
+        raise ValueError('''
+*
+*   The 'mode' options are 'minmax', 'zscore', or 'pct_rank', you gave me
+*   {0}.
+*
+'''.format(mode))
+
+    return tsutils.print_input(print_input, otsd, tsd, '_{0}'.format(mode),
+                               float_format=float_format)
+
+
 def main():
     if not os.path.exists('debug_tstoolbox'):
         sys.tracebacklimit = 0
